﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Networking;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;

namespace DroneTest
{
    public static class DroneControl
    {
        private static DatagramSocket udpSocket;
        private static DataWriter udpWriter;

        private static volatile uint sequenceNumber = 1;
        private static volatile bool stop = false;

        public static bool StubMode { get; set; }

        public static async Task ConnectAsync()
        {
            if (!StubMode)
            {
                // Set up the UDP connection
                var remotePort = "5556";
                var droneIP = new HostName("192.168.1.1");

                udpSocket = new DatagramSocket();
                await udpSocket.BindServiceNameAsync(remotePort);
                await udpSocket.ConnectAsync(droneIP, remotePort);
                udpWriter = new DataWriter(udpSocket.OutputStream);

                udpWriter.WriteByte(1);
                await udpWriter.StoreAsync();
            }

            var loop = Task.Run(() => DroneLoop());
        }

        // destroy the connection
        public static void Close()
        {
            if (!StubMode)
            {
                udpSocket.Dispose();

                udpSocket = null;
                udpWriter = null;
            }

            Stop();
        }

        private static async Task DroneLoop()
        {
            while (true)
            {
                if (stop)
                {
                    stop = false;
                    sequenceNumber = 1;
                    break;
                }

                var commandToSend = DroneState.GetNextCommand(sequenceNumber);
                await SendCommandAsync(commandToSend);

                sequenceNumber++;
                await System.Threading.Tasks.Task.Delay(30);
            }
        }

        // Send a command to the drone
        private static async Task SendCommandAsync(string command)
        {
            Debug.WriteLine("Command: " + command);

            if (!StubMode)
            {
                udpWriter.WriteString(command);
                await udpWriter.StoreAsync();
            }
        }

        // Suspends the control loop and resets the sequence number counter
        public static void Stop()
        {
            stop = true;
        }
    }
}
