﻿using BluetoothConnection.Objects;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Storage.Streams;

namespace BluetoothConnection.Common
{
    public static class ConnectionExtensions
    {
        private const uint READ_LEN = 4;

        public static async Task WriteMessageAsync(this DataWriter writer, string message)
        {
            writer.WriteString(message);
            await writer.StoreAsync();
        }

        public static async Task WriteMessageAsync(this DataWriter writer, Request request)
        {
            var message = JsonConvert.SerializeObject(request);
            await writer.WriteMessageAsync(message);
        }

        public static async Task<string> ReadMessageAsync(this DataReader reader)
        {
            try
            {
                var bytesRead = await reader.LoadAsync(READ_LEN);
                if (bytesRead > 0)
                {
                    var lenString = reader.ReadString(READ_LEN);
                    var len = uint.Parse(lenString);

                    bytesRead = await reader.LoadAsync(len);
                    if (bytesRead > 0)
                    {
                        var response = reader.ReadString(len);
                        return response;
                    }
                }
            }
            catch
            { }

            return null;
        }

        public static async Task<T> ReadMessageAsync<T>(this DataReader reader)
        {
            var responseData = await reader.ReadMessageAsync();
            var response = JsonConvert.DeserializeObject<T>(responseData);

            return response;
        }
    }
}
