﻿using BluetoothConnection.Objects;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Devices.Bluetooth.Rfcomm;
using Windows.Devices.Enumeration;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;
using BluetoothConnection.Common;

namespace BluetoothConnection.Services
{
    public class BluetoothRfcomm : IRfcomm, IDisposable
    {
        private RfcommDeviceService service;
        private StreamSocket socket;

        private DataWriter writer;
        private DataReader reader;

        public bool IsConnected { get; set; }

        public async Task ConnectAsync(string deviceName)
        {
            var deviceInfoCollection = await DeviceInformation.FindAllAsync(
                RfcommDeviceService.GetDeviceSelector(RfcommServiceId.SerialPort));

            var device = deviceInfoCollection.Where(d => d.Name == deviceName).FirstOrDefault();
            if (device != null)
            {
                try
                {
                    service = await RfcommDeviceService.FromIdAsync(device.Id);

                    if (socket != null)
                    {
                        // Disposing the socket with close it and release all resources associated with the socket
                        socket.Dispose();
                    }

                    socket = new StreamSocket();
                    await socket.ConnectAsync(service.ConnectionHostName,
                                              service.ConnectionServiceName);

                    writer = new DataWriter(socket.OutputStream);
                    reader = new DataReader(socket.InputStream);

                    IsConnected = true;
                }
                catch
                {
                    socket.Dispose();
                    socket = null;

                    throw;
                }
            }
        }

        public async Task<IEnumerable<Song>> GetSongsListAsync()
        {
            var request = new Request { RequestType = RequestType.List };
            await writer.WriteMessageAsync(request);

            var response = await reader.ReadMessageAsync<Response>();
            if (response != null && response.ResponseType == ResponseType.List)
            {
                var songs = JsonConvert.DeserializeObject<List<Song>>(response.Arguments.ToString());
                return songs;
            }

            return null;
        }

        public async Task PlaySongAsync(Song song)
        {
            var request = new Request
            {
                RequestType = RequestType.Play,
                Arguments = song.ID
            };

            await writer.WriteMessageAsync(request);
        }

        public async Task StopMusicAsync()
        {
            var request = new Request { RequestType = RequestType.Stop };
            await writer.WriteMessageAsync(request);
        }

        public void Dispose()
        {
            try
            {
                if (reader != null)
                {
                    reader.DetachStream();
                    reader.Dispose();
                    reader = null;
                }
            }
            catch
            { }

            try
            {
                if (writer != null)
                {
                    writer.DetachStream();
                    writer.Dispose();
                    writer = null;
                }
            }
            catch
            { }

            try
            {
                if (socket != null)
                {
                    socket.Dispose();
                    socket = null;
                }
            }
            catch
            { }
        }
    }
}
