using System;
using BluetoothConnection.Services;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using MvvmLightExtensions.WindowsStore.ViewModels;
using MvvmLightExtensions.WindowsStore.Services;
using System.Collections.Generic;
using BluetoothConnection.Objects;

namespace BluetoothConnection.ViewModels
{
    public class MainViewModel : BindableBase
    {
        private IRfcomm communicator;
        private IMessageService messageService;

        private IEnumerable<Song> songs;
        public IEnumerable<Song> Songs
        {
            get
            {
                return songs;
            }
            set
            {
                base.SetProperty(ref songs, value);
            }
        }

        public RelayCommand ConnectCommand { get; set; }
        public RelayCommand GetSongsListCommand { get; set; }
        public RelayCommand StopMusicCommand { get; set; }
        public RelayCommand<Song> ItemSelectedCommand { get; set; }

        public MainViewModel(IRfcomm communicator, IMessageService messageService)
        {
            this.communicator = communicator;
            this.messageService = messageService;

            this.CreateCommnds();
        }

        private void CreateCommnds()
        {
            ConnectCommand = new RelayCommand(async () =>
                {
                    try
                    {
                        await communicator.ConnectAsync("Gadgeteer");
                    }
                    catch
                    { }

                    if (!communicator.IsConnected)
                    {
                        // Se il device non si  connesso, si  verificato un errore.
                        await messageService.ShowAsync("Impossibile effettuare la connessione.", "Errore");
                    }
                    else
                    {
                        ConnectCommand.RaiseCanExecuteChanged();
                        GetSongsListCommand.RaiseCanExecuteChanged();
                        StopMusicCommand.RaiseCanExecuteChanged();

                        GetSongsListCommand.Execute(null);
                    }
                }, () =>
                {
                    return !communicator.IsConnected;
                });

            GetSongsListCommand = new RelayCommand(async () =>
                {
                    var error = false;
                    try
                    {
                        Songs = await communicator.GetSongsListAsync();
                    }
                    catch
                    { error = true; }

                    if (error)
                    {
                        await messageService.ShowAsync("Errore durante l'esecuzione del comando.", "Errore");
                    }
                }, () =>
                {
                    return communicator.IsConnected;
                });

            ItemSelectedCommand = new RelayCommand<Song>(async (song) =>
                {
                    var error = false;
                    try
                    {
                        await communicator.PlaySongAsync(song);
                    }
                    catch
                    {
                        error = true;
                    }

                    if (error)
                    {
                        await messageService.ShowAsync("Errore durante l'esecuzione del comando.", "Errore");
                    }
                });

            StopMusicCommand = new RelayCommand(async () =>
                {
                    var error = false;
                    try
                    {
                        await communicator.StopMusicAsync();
                    }
                    catch
                    {
                        error = true;
                    }

                    if (error)
                    {
                        await messageService.ShowAsync("Errore durante l'esecuzione del comando.", "Errore");
                    }
                }, () =>
                {
                    return communicator.IsConnected;
                });
        }
    }
}