﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MvvmLightExtensions.WindowsStore.Sharing;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Ioc;
using Microsoft.Practices.ServiceLocation;

namespace MvvmLightExtensions.WindowsStore.ViewModels
{
    public abstract class ViewModelLocatorBase
    {
        protected void Register<TInterface, TClass>(bool createInstanceImmediately = false)
            where TInterface : class
            where TClass : class
        {
            if (!SimpleIoc.Default.IsRegistered<TInterface>())
                SimpleIoc.Default.Register<TInterface, TClass>(createInstanceImmediately);
        }

        protected void Register<TClass>(bool createInstanceImmediately = false)
            where TClass : class
        {
            if (!SimpleIoc.Default.IsRegistered<TClass>())
                SimpleIoc.Default.Register<TClass>(createInstanceImmediately);
        }

        protected void Register<TClass>(Func<TClass> factory)
            where TClass : class
        {
            if (!SimpleIoc.Default.IsRegistered<TClass>())
                SimpleIoc.Default.Register<TClass>(factory);
        }

        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        public ViewModelLocatorBase()
        {
            ServiceLocator.SetLocatorProvider(() => SimpleIoc.Default);

            //if (ViewModelBase.IsInDesignModeStatic)
            //{
            //    // Create design time view services and models
            //    SimpleIoc.Default.Register<IDataService, DesignDataService>();
            //}
            //else
            //{
            //    // Create run time view services and models
            //    SimpleIoc.Default.Register<IDataService, DataService>();
            //}
        }

        public static void RegisterSharingService()
        {
            SimpleIoc.Default.Register<SharingService>(createInstanceImmediately: true);
        }
    }
}
