﻿using System;
using System.Collections;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Presentation.Shapes;
using Microsoft.SPOT.Touch;

using Gadgeteer.Networking;
using GT = Gadgeteer;
using GTM = Gadgeteer.Modules;
using Gadgeteer.Modules.GHIElectronics;
using HomeAutomation.Objects;

namespace HomeAutomation
{
    public partial class Program
    {
        private Bluetooth.Client client;

        // This method is run when the mainboard is powered up or reset.   
        void ProgramStarted()
        {
            multicolorLed.GreenBlueSwapped = true;
            client = bluetooth.ClientMode;
            music.SetVolume(255);

            bluetooth.SetDeviceName("Gadgeteer");
            bluetooth.SetPinCode("1234");
            bluetooth.DataReceived += new Bluetooth.DataReceivedHandler(bluetooth_DataReceived);
            client.EnterPairingMode();

            // Use Debug.Print to show messages in Visual Studio's "Output" window during debugging.
            multicolorLed.TurnGreen();
            Debug.Print("Program Started");
        }

        private void bluetooth_DataReceived(Bluetooth sender, string data)
        {
            // Controlla il tipo di richiesta ricevuta.
            Request message = null;

            try
            {
                message = (Request)DataContractJsonSerializer.Deserialize(data);
            }
            catch { }

            if (message == null)
                return;

            if (message.RequestType == RequestType.List)
            {
                var songsList = new ArrayList();

                songsList.Add(new Song 
                { 
                    ID = Songs.DoctorWho, 
                    Name = "Doctor Who Theme",
                    Image = "http://www.bbcamerica.com/doctor-who/files/2013/10/Doctorwho_50th-anniversary_thumbnail_02.jpg"
                });               
                
                songsList.Add(new Song 
                { 
                    ID = Songs.TheBigBangTheory, 
                    Name = "The Bing Bang Theory Theme",
                    Image = "http://upload.wikimedia.org/wikipedia/en/1/1b/BigBangTheoryTitleCard.png"
                });

                songsList.Add(new Song
                {
                    ID = Songs.StarWars,
                    Name = "Star Wars Theme",
                    Image = "http://upload.wikimedia.org/wikipedia/commons/thumb/6/6c/Star_Wars_Logo.svg/694px-Star_Wars_Logo.svg.png"
                });

                var responseData = new Response { ResponseType = ResponseType.List, Arguments = songsList };
                var response = DataContractJsonSerializer.Serialize(responseData);

                this.SendMessage(response);
            }
            else if (message.RequestType == RequestType.Play)
            {
                var id = Convert.ToInt32(message.Arguments.ToString());
                this.StartPlaying(id);
            }
            else if (message.RequestType == RequestType.Stop)
            {
                music.StopPlaying();
            }
        }

        #region Helpers

        private void SendMessage(string message)
        {
            var len = message.Length.ToString().PadLeft(4, '0');
            client.Send(len);
            client.Send(message);
        }

        private void StartPlaying(int id)
        {
            switch (id)
            {
                case Songs.DoctorWho:
                    music.Play(Resources.GetBytes(Resources.BinaryResources.DoctorWho));
                    break;

                case Songs.TheBigBangTheory:
                    music.Play(Resources.GetBytes(Resources.BinaryResources.TBBT));
                    break;

                case Songs.StarWars:
                    music.Play(Resources.GetBytes(Resources.BinaryResources.StarWars));
                    break;

                default:
                    break;
            }
        }

        #endregion
    }
}
