using System;
using Microsoft.SPOT;
using System.Text;

namespace HomeAutomation
{
    public static class StringExtensions
    {
        /// <summary>
        /// Returns a new string of a specified length in which the beginning of the current string is padded with spaces or with a specified character.
        /// </summary>
        /// <param name="original"></param>
        /// <param name="totalWidth">The number of characters in the resulting string, equal to the number of original characters plus any additional padding characters.</param>
        /// <param name="paddingChar">The padding character.</param>
        /// <returns></returns>
        public static string PadLeft(this string original, int totalWidth, char paddingChar = ' ')
        {
            return PadHelper(original, totalWidth, paddingChar, true);
        }

        /// <summary>
        /// Returns a new string of a specified length in which the end of the current string is padded with spaces or with a specified character.
        /// </summary>
        /// <param name="original"></param>
        /// <param name="totalWidth">The number of characters in the resulting string, equal to the number of original characters plus any additional padding characters.</param>
        /// <param name="paddingChar">The padding character.</param>
        /// <returns></returns>
        public static string PadRight(this string original, int totalWidth, char paddingChar = ' ')
        {
            return PadHelper(original, totalWidth, paddingChar, false);
        }

        /// <summary>
        /// Private method to do the actual padding.
        /// </summary>
        /// <param name="original"></param>
        /// <param name="totalWidth"></param>
        /// <param name="paddingChar"></param>
        /// <param name="appendLeft"></param>
        /// <returns></returns>
        private static string PadHelper(string original, int totalWidth, char paddingChar, bool appendLeft)
        {
            // this seems to be quicker than a simple while (length < totalWidth) pad
            if (original.Length >= totalWidth)
                return original;

            var result = new StringBuilder(totalWidth);
            int padCount = totalWidth - original.Length;
            for (int i = 0; i < padCount; i++)
                result.Append(paddingChar);
            if (appendLeft)
                return result + original;
            return original + result;
        }
    }
}
